import os
import cv2
import numpy as np
import sklearn
from sklearn import *
import time
from pythonosc.udp_client import SimpleUDPClient
import threading

def pdopen(patch):

    # Set your pd path below (for Mac)
    pd_path = '/Applications/Pd-0.51-4.app'

    if os.sys.platform == 'darwin':
        pd_executable = pd_path + '/Contents/Resources/bin/pd' #for Mac
    elif os.sys.platform == 'win32':
        pd_executable = '"C:\\Program Files\\Pd\\bin\\pd.exe"' #for Windows

    pd_patch = patch
    command = pd_executable + ' -open ' + pd_patch + ' -nogui'
    os.popen(command)
    

def posture_generator():

    # capture the webcam frames
    webcam = cv2.VideoCapture(0)

    print("""INSTRUCTIONS:\nKeep pressing the following keys to write frames.
    \nFirst posture: keep pressing "q" to write frames.
    Second posture: keep pressing "w" to write frames.
    Third posture: keep pressing "e" to write frames.
    Fourth posture: keep pressing "r" to write frames.
    Fifth posture: keep pressing "t" to write frames.
    \nPress "space" to close the posture_generator().
    """)
    
    # create empty lists to store the posture's frames
    p1, p2, p3, p4, p5 = [], [], [], [], []
    labels = []
    
    v, w, x, y, z = 0, 0, 0, 0, 0

    while True:

        ret, frame = webcam.read()

        # resize webcam frame by 10x10 pixels
        frame = cv2.resize(frame, (100, 100), interpolation=cv2.INTER_NEAREST)
        # convert webcam frame to grayscale
        frame = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
        # create a window with a suitable name and size to display frame on the screen
        cv2.namedWindow('video', cv2.WINDOW_NORMAL)
        # flip the webcam frame
        flipHorizontal = cv2.flip(frame, 1)
        # show the webcam frame
        cv2.imshow('video', flipHorizontal)

        # keep pressing q to write the first posture frame 
        if cv2.waitKey(1) == ord('q'):
            print(f'Number of recorded frames for first posture: {v}')
            p1.append(frame.flatten())
            labels.append(1)
            v += 1

        # keep pressing w to write the second posture frames 
        if cv2.waitKey(1) == ord('w'):
            print(f'Number of recorded frames for second posture: {w}')
            p2.append(frame.flatten())
            labels.append(2)
            w += 1

        # keep pressing e to write the third posture frames 
        if cv2.waitKey(1) == ord('e'):
            print(f'Number of recorded frames for third posture: {x}')
            p3.append(frame.flatten())
            labels.append(3)
            x += 1
            
        # keep pressing r to write the fourth posture frames 
        if cv2.waitKey(1) == ord('r'):
            print(f'Number of recorded frames for fourth posture: {y}')
            p4.append(frame.flatten())
            labels.append(4)
            y += 1
            
        # keep pressing r to write the fifth posture frames 
        if cv2.waitKey(1) == ord('t'):
            print(f'Number of recorded frames for fifth posture: {z}')
            p5.append(frame.flatten())
            labels.append(5)
            z += 1

        key = cv2.waitKey(1)
        # keep pressing space to break 
        if key == 32:
            break

    webcam.release()
    cv2.destroyAllWindows()

    rawdata = np.concatenate((p1, p2, p3, p4, p5))
    labels = np.array(labels)
    
    return rawdata, labels

def player():
    
    webcam = cv2.VideoCapture(0)

    print("""INSTRUCTIONS:\nPlay with the postures you just recorded and make some sound!
    \nPress "Space" to close the player().
    """)
    
    while True:
        ret, frame = webcam.read()

        # resize webcam frame by 10x10 pixels
        frame = cv2.resize(frame, (100, 100), interpolation=cv2.INTER_NEAREST)
        # convert webcam frame to grayscale
        frame = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
        # create a window with a suitable name and size to display frame on the screen
        cv2.namedWindow('video', cv2.WINDOW_NORMAL)
        # flip the webcam frame
        flipHorizontal = cv2.flip(frame, 1)
        # show the webcam frame
        cv2.imshow('video', flipHorizontal)

        frame = frame.flatten().reshape(1, -1)
        predict = svm.predict(frame)
        prediction = int(predict)
        print(prediction)
        client.send_message('/prediction', prediction)
        
        key = cv2.waitKey(1)
        # keep pressing space to break 
        if key == 32:
            client.send_message('/prediction', 6)
            break

    webcam.release()
    cv2.destroyAllWindows()

# RUNNING THE SCRIPT
if __name__ == "__main__":

    # run posture_generator for training
    rawdata, labels = posture_generator()

    #creating train/test split
    rawdata_train, rawdata_test, lab_train, lab_test = sklearn.model_selection.train_test_split(rawdata, labels, test_size=0.2)
    #training the model
    svm = sklearn.svm.SVC(kernel='linear', C=1)
    svm.fit(rawdata_train, lab_train)
    #applying the the model on the test data (features)
    lab_predict =  svm.predict(rawdata_test)

    # declare pd patch to use
    pd_patch = '.\superawesomepatch.pd'
    # open pd on thread so that main continues, passing patch as argument
    t1 = threading.Thread(target=pdopen, args=(pd_patch,))
    t1.start()
    # declare client on local host
    client = SimpleUDPClient('127.0.0.1', 8006)
    # run player
    player()





